"use strict";


/**
 * ParentalControl interface
 * @class ParentalControl
 * @namespace PluginInterface
 * @static
 */
PluginInterface.ParentalControl = new function() {
    /*
     ***********************************************************************************************************
     PRIVATE VARIABLES AND FUNCTIONS
     ***********************************************************************************************************
     */
    /**
     * Current PIN code
     * @private
     * @type {String}
     * @property _currPinCode
     * @default "0000"
     */
    var _currPinCode = "0000",
    /**
     * Age lock status
     * @private
     * @type {Number}
     * @property _ageLock
     * @default 0 = Off
     */
    _ageLock = 0,
    /**
     * Program lock status
     * @private
     * @type {Number}
     * @property _lockProgram
     * @default 0 = Off
     */
    _lockProgram = 0,
    /**
     * Menu lock status
     * @private
     * @type {Number}
     * @property _menuLock
     * @default 0 = Off
     */
    _menuLock = 0,
    /**
     * Pin prompt enter function
     * @method _callback_pin_enter
     * @private
     */
    _callback_pin_enter = null,
    /**
     /**
     * Pin prompt remove function
     * @method _callback_pin_remove
     * @private
     */
    _callback_pin_remove = null,
    /**
     * Parental control callback from plugin
     * 0 = channel, 1 = age, 2 = time
     * @private
     * @method _fCallbackParental
     * @param lockType {Number} Lock type
     * @param lockStatus {Number} Lock status
     */
    _fCallbackParental = function (lockType, lockStatus, age, serviceIndex) {
        var activeList = PluginInterface.ServiceZapper.fGetActiveServiceList();
        var activeIndex = PluginInterface.ServiceZapper.fGetActiveService(activeList);
        var data = PluginInterface.ServiceZapper.fGetServiceData(activeList, activeIndex);
        if (serviceIndex == data.index) {
            switch (lockType) {
                case this.eLockType.CHANNEL:
                    if (lockStatus && (lockStatus == data.isLocked)) {
                        if (PluginInterface.ChannelInstallation.fGetIsScanning() == 0) {
                            _callback_pin_enter(this.eLockType.CHANNEL);
                        }
                    }
                    else {
                        _callback_pin_remove(this.eLockType.CHANNEL);
                    }
                    break;
                case this.eLockType.AGE:
                    if (PluginInterface.ChannelInstallation.fGetIsScanning() == 0) {
                        var ageLimit = this.eAgeLock.OFF;
                        var dateTime = PluginInterface.Setup.fGetTimeDate().split(":");
                        var hour = parseInt(dateTime[3], 10);

                        if ((hour >= 6) && (hour < 22)) {
                            ageLimit = this.eAgeLock.AGE_15;
                        }
                        else if ((hour >= 22) && (hour < 23)) {
                            ageLimit = this.eAgeLock.AGE_16;
                        }
                        else if ((hour >= 23) || (hour < 6)) {
                            ageLimit = this.eAgeLock.AGE_18;
                        }

                        if (parseInt(age, 10) <= ageLimit) {
                            _callback_pin_remove(this.eLockType.AGE);
                        }
                        else {
                            _callback_pin_enter(this.eLockType.AGE);
                        }
                    }
                    break;
                case this.eLockType.TIME:
                    break;
            }
        }
        else {
            _callback_pin_remove();
        }
    };
    /*
     ***********************************************************************************************************
     PRIVILEGED PROPERTIES AND METHODS
     ***********************************************************************************************************
     */
    /**
     * Enumerated PIN source values
     * @property ePinSource
     * @type {{ERROR: number, POPUP: number, CHANNEL_LIST: number, MENU: number, SETTINGS: number}}
     * @final
     */
    this.ePinSource = {
        ERROR: 0,
        POPUP: 1,
        CHANNEL_LIST: 2,
        MENU: 3,
        SETTINGS: 4
    };

    /**
     * Enumerated lock type values
     * @property eLockType
     * @type {{CHANNEL: number, AGE: number, TIME: number, PVR: number, PVR_AGE: number}}
     * @final
     */
    this.eLockType = {
        CHANNEL: 0,
        AGE: 1,
        TIME: 2,
        PVR: 3,
        PVR_AGE: 4
    };

    /**
     * Enumerated age lock values
     * @property eAgeLock
     * @type {{OFF: number, AGE_3: number, AGE_4: number, AGE_5: number, AGE_6: number, AGE_7: number, AGE_8: number, AGE_9: number, AGE_10: number, AGE_11: number, AGE_12: number, AGE_13: number, AGE_14: number, AGE_15: number, AGE_16: number, AGE_17: number, AGE_18: number}}
     * @final
     */
    this.eAgeLock = {
        OFF: 0,
        AGE_3: 3,
        AGE_4: 4,
        AGE_5: 5,
        AGE_6: 6,
        AGE_7: 7,
        AGE_8: 8,
        AGE_9: 9,
        AGE_10: 10,
        AGE_11: 11,
        AGE_12: 12,
        AGE_13: 13,
        AGE_14: 14,
        AGE_15: 15,
        AGE_16: 16,
        AGE_17: 17,
        AGE_18: 18
    };

    /**
     * Enumerated lock program values
     * @property eLockProgram
     * @type {{OFF: number, ON: number}}
     * @final
     */
    this.eLockProgram = {
        OFF: 0,
        ON: 1
    };

    /**
     * Enumerated menu lock values
     * @property eMenuLock
     * @type {{OFF: number, ON: number}}
     * @final
     */
    this.eMenuLock = {
        OFF: 0,
        ON: 1
    };
    /**
     * File name to store lock program value
     * @type {String}
     * @property DATA_UI_LOCK_PROGRAM
     * @final
     */
    this.DATA_UI_LOCK_PROGRAM = "ui_lock_program";
    /**
     * File name to store menu lock value
     * @type {String}
     * @property DATA_UI_MENU_LOCK
     * @final
     */
    this.DATA_UI_MENU_LOCK = "ui_menu_lock";

    /**
     * Initialize Parental control module
     * @method fInitParentalControl
     * @static
     */
    this.fInitParentalControl = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.parentalNotification = _fCallbackParental.bind(this);
        PluginInterface.Setup.stb.registerParentalCallback("");

        var value = parseInt(PluginInterface.Setup.stb.loadData(this.DATA_UI_LOCK_PROGRAM), 10);
        value = (typeof value == 'undefined' || isNaN(value)) ? _lockProgram : value;
        PluginInterface.Setup.stb.setChannelLockEnable(this.DATA_UI_LOCK_PROGRAM, "" + value);
    };

    /**
     * Set Callbacks
     * @method fSetCallbacks
     * @param pinprompt {Function} Show pin input
     * @param promptremove {Function} Remove pin input
     * @static
     */
    this.fSetCallbacks = function (pinprompt, promptremove) {
        _callback_pin_enter = pinprompt;
        _callback_pin_remove = promptremove;
    };

    /**
     * Check PIN code
     * @method fCheckPinCode
     * @param pinCode {String} PIN code to be checked
     * @returns {boolean} True/False
     * @static
     */
    this.fCheckPinCode = function (pinCode, source, service) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return (_currPinCode == pinCode);
        }
        return PluginInterface.Setup.stb.checkPinCode("" + pinCode, "" + source, "" + service) == 1;
    };

    /**
     * Check Smart Card PIN code
     * @method fCheckCakPinCode
     * @param pinCode {String} Smart Card PIN code to be checked
     * @returns {Boolean}
     * @async
     * @static
     */
    this.fCheckCakPinCode = function (pinCode) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return (_currPinCode == pinCode);
        }
        var result = PluginInterface.Setup.stb.cakPinCode("" + PluginInterface.Cak.pinType.PCT_PARENTAL_CONTROL, "" + pinCode);
        return 1;
    };
    /**
     * Set PIN code
     * @method fSetPinCode
     * @param pinCode {String} PIN code to be set
     * @static
     */
    this.fSetPinCode = function (pinCode) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _currPinCode = pinCode;
            return;
        }
        PluginInterface.Setup.stb.setPinCode("" + pinCode);
    };

    /**
     * Plays locked active service after correct PIN code
     * @method fPlayLockedActiveService
     * @static
     */
    this.fPlayLockedActiveService = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.playLockedActiveService("");
    };

    /**
     * Set Age lock
     * @method fSetAgeLock
     * @param age {Number} Age to set
     * @static
     */
    this.fSetAgeLock = function (age) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _ageLock = age;
            return;
        }
        PluginInterface.Setup.stb.setParentalRate("" + age);
    };

    /**
     * Get Age lock
     * @method fGetAgeLock
     * @returns {String} Age lock
     * @static
     */
    this.fGetAgeLock = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _ageLock;
        }
        return PluginInterface.Setup.stb.getParentalRate("");
    };

    /**
     * Set Service lock
     * @method fSetServiceLock
     * @param listIndex {Number} Service list index
     * @param serviceIndex {Number} Service index
     * @param isLocked {Boolean} Is channel locked
     * @static
     */
    this.fSetServiceLock = function (listIndex, serviceIndex, isLocked) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        isLocked = isLocked ? 1 : 0;
        PluginInterface.Setup.stb.setChannelLock("" + listIndex, "" + serviceIndex, "" + isLocked);
    };

    /**
     * Get Service lock: true = channel locked, false = channel unlocked
     * @method fGetServiceLock
     * @param listIndex {Number} Service list index
     * @param serviceIndex {Number} Service index
     * @returns {Boolean} true = channel locked, false = channel unlocked
     * @static
     */
    this.fGetServiceLock = function (listIndex, serviceIndex) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return PluginInterface.ServiceZapper.fGetServiceData(listIndex, serviceIndex).isLocked;
        }
        return PluginInterface.Setup.stb.getChannelLock("" + listIndex, "" + serviceIndex);
    };

    /**
     * Returns 1 if parental control is available, else returns 0
     * @method fGetAgeLockingSupportStatus
     * @returns {number} 1 = parental control is available, 0 = parental control not available
     * @static
     */
    this.fGetAgeLockingSupportStatus = function () {
        return 1;
    };

    /**
     * Set Lock program value
     * @method fSetLockProgram
     * @param value {Number} Lock program value
     * @static
     */
    this.fSetLockProgram = function (value) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _lockProgram = value;
            return;
        }
        PluginInterface.Setup.stb.setChannelLockEnable(this.DATA_UI_LOCK_PROGRAM, value + "");
    };

    /**
     * Get Lock program value
     * @method fGetLockProgram
     * @returns {Number} Lock program value
     * @static
     */
    this.fGetLockProgram = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _lockProgram;
        }
        var value = parseInt(PluginInterface.Setup.stb.getChannelLockEnable(""), 10);
        value = (typeof value == 'undefined' || isNaN(value)) ? _lockProgram : value;
        return value;
    };

    /**
     * Set Menu lock value
     * @method fSetMenuLock
     * @param value {Number} Menu lock value
     * @static
     */
    this.fSetMenuLock = function (value) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _menuLock = value;
            return;
        }
        PluginInterface.Setup.stb.saveData(this.DATA_UI_MENU_LOCK, value + "");
    };

    /**
     * Get Menu lock value
     * @method fGetMenuLock
     * @returns {Number} Menu lock value
     * @static
     */
    this.fGetMenuLock = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _menuLock;
        }
        var value = parseInt(PluginInterface.Setup.stb.loadData(this.DATA_UI_MENU_LOCK), 10);
        value = (typeof value == 'undefined' || isNaN(value)) ? _menuLock : value;
        return value;
    };
};