"use strict";

/**
 * Pvr interface
 * @class Pvr
 * @namespace PluginInterface
 * @static
 */
PluginInterface.Pvr = new function() {
    /*
     ***********************************************************************************************************
     PRIVATE VARIABLES AND FUNCTIONS
     ***********************************************************************************************************
     */
    /**
     * PVR state
     * 0 = Live, 1 = Paused, 2 = Play, 3 = rw, 4 = ff
     * @private
     * @type {Number}
     * @property _PVRState
     * @default 0 = LIVE
     */
    var _PVRState = 0,
    /**
     * Previous PVR state
     * @property _PVRstate_prev
     * @type {Number}
     * @private
     * @default 0 = LIVE
     */
    _PVRstate_prev = 0,
    /**
     * PVR event
     * @property _PVREvent
     * @type {Number}
     * @private
     * @default 0
     */
    _PVREvent = 0,
    /**
     * PVR callback
     * @method _callback_pvr_function
     * @private
     */
    _callback_pvr_function = null,
    /**
     * PVR browsing callback
     * @method _callback_pvr_browsing_function
     * @private
     */
    _callback_pvr_browsing_function = null,
    /**
     * PVR recording state change callback
     * @method _callback_pvr_record_state_change
     * @private
     */
    _callback_pvr_record_state_change = null,
    /**
     * PVR play record state change callback
     * @method _callback_play_record_state_change
     * @private
     */
    _callback_play_record_state_change = null,
    /**
     * PVR schedule recording callback
     * @method _callback_pvr_scheduled_recording
     * @private
     */
    _callback_pvr_scheduled_recording = null,
    /**
     * Current speed
     * @property _currentSpeed
     * @type {Number}
     * @private
     * @default 1
     */
    _currentSpeed = 1,
    /**
     * Duration of recorded stream
     * @property _endTimeshiftTime
     * @type {Number}
     * @private
     * @default 0
     */
    _endTimeshiftTime = 0,
    /**
     * Current time of recorded stream
     * @property _currentTimeshiftTime
     * @type {Number}
     * @private
     * @default 0
     */
    _currentTimeshiftTime = 0,
    /**
     * Transition flag
     * @property _transitionToRecord
     * @type {number}
     * @private
     * @default 0
     */
    _transitionToRecord = 0,
    /**
     * Scheduled record index
     * @property _scheduledRecordIndex
     * @type {Number}
     * @private
     * @default 0
     */
    _scheduledRecordIndex = 0,
    /**
     * PVR items
     * @property _pvrArray
     * @type {Array}
     * @private
     */
    _pvrArray = [],
    /**
     * PVR count
     * @property _pvrCount
     * @type {Number}
     * @private
     * @default 0
     */
    _pvrCount = 0,
    /**
     * PVR timeout ID
     * @property _tPvrCallback
     * @type {Number}
     * @private
     */
    _tPvrCallback,
    /**
     * Current PVR playing index
     * @property _currentPvrPlayingIndex
     * @type {Number}
     * @private
     * @default -1
     */
    _currentPvrPlayingIndex = -1,
    /**
     * PVR padding before
     * @property _pvrPaddingBefore
     * @type {Number}
     * @private
     * @default 0
     */
    _pvrPaddingBefore = 0,
    /**
     * PVR padding after
     * @property _pvrPaddingAfter
     * @type {Number}
     * @private
     * @default 0
     */
    _pvrPaddingAfter = 0,
    /**
     * Schedules count
     * @property _schedulesCount
     * @type {Number}
     * @private
     * @default 5
     */
    _schedulesCount = 5;
    /**
     * Constructor for PVR object
     * @method _getPVR
     * @private
     * @param strDesc {String} Description
     * @param index {Number}
     * @returns {{name: *, type: (number|*), description: *, duration: *, genre: string, parental: number, index: *, sttmhour: *, sttmmin: *, sttmsec: *, ettmhour: *, ettmmin: *, ettmsec: *, drtmhour: *, drtmmin: *, drtmsec: *, size: *, lock: *, parentalRating: * }}
     */
    var _getPVR = function (strDesc, index) {
        var pvr = strDesc.split("|");
        var startTime = (parseInt(pvr[3], 10) < 10 ? ('0' + parseInt(pvr[3], 10)) : parseInt(pvr[3], 10)) + ':' + (parseInt(pvr[4], 10) < 10 ? ('0' + parseInt(pvr[4], 10)) : parseInt(pvr[4], 10)) + ':' + (parseInt(pvr[5], 10) < 10 ? ('0' + parseInt(pvr[5], 10)) : parseInt(pvr[5], 10));
        var endTime = (parseInt(pvr[6], 10) < 10 ? ('0' + parseInt(pvr[6], 10)) : parseInt(pvr[6], 10)) + ':' + (parseInt(pvr[7], 10) < 10 ? ('0' + parseInt(pvr[7], 10)) : parseInt(pvr[7], 10)) + ':' + (parseInt(pvr[8], 10) < 10 ? ('0' + parseInt(pvr[8], 10)) : parseInt(pvr[8], 10));
        var duration = (parseInt(pvr[9], 10) < 10 ? ('0' + parseInt(pvr[9], 10)) : parseInt(pvr[9], 10)) + ':' + (parseInt(pvr[10], 10) < 10 ? ('0' + parseInt(pvr[10], 10)) : parseInt(pvr[10], 10)) + ':' + (parseInt(pvr[11], 10) < 10 ? ('0' + parseInt(pvr[11], 10)) : parseInt(pvr[11], 10));
        return {
            name: pvr[0],
            type: PluginInterface.MediaBrowsing.fileType.PVR_VIDEO,
            description: pvr[1],
            genre: "No genre",
            parental: 0,
            index: index,
            url: index,
            start_time: startTime,
            end_time: endTime,
            duration: duration,
            size: pvr[12],
            lock: parseInt(pvr[13]),
            parentalRating: parseInt(pvr[14]),
            sourceList: PluginInterface.Pvr.pvrSourceList.TV//parseInt(pvr[15], 10)
        }
    };
    /**
     * PVR callback from plugin
     * @private
     * @method _fCallbackPvr
     * @param event {Number}
     * @param pData {Number}
     * @param pData2 {Number}
     * @param pData3 {Number}
     * @param expirationDate {String}
     */
    var _fCallbackPvr = function (event, pData, pData2, pData3, expirationDate) {
        //PluginInterface.Tools.debug('EVENT: ' + event + ' | DATA1: ' + pData + ' | DATA2:' + pData2 + ' | DATA3:' + pData3);
        switch (event) {
            case this.ePvrEvent.RECORD_START:
                _transitionToRecord = 0;
                _scheduledRecordIndex = pData;
                if (_callback_pvr_record_state_change) {
                    _callback_pvr_record_state_change(event, pData);
                }
                break;
            case this.ePvrEvent.RECORD_STOP:
                // Re-initialize _transitionRecord if STOP occurs before MAL_PVR_EVENT_RECORD_START
                // is received.
                _transitionToRecord = 0;
                setTimeout(function () {
                    PluginInterface.Hbbtv.fSetHbbtvOn();
                    PluginInterface.Epg.fRegisterEpgEventCallback();
                }, 1000);
                if (_callback_pvr_record_state_change) {
                    _callback_pvr_record_state_change(event, pData);
                }
                return;
            case this.ePvrEvent.RECORD_POSITION:
                _endTimeshiftTime = pData;
                break;
            case this.ePvrEvent.RECORD_ADD:
                if (_callback_pvr_record_state_change) {
                    _callback_pvr_record_state_change(event, pData, pData2);
                }
                break;
            case this.ePvrEvent.RECORD_RESOURCE_ISSUE:
                _transitionToRecord = 0;
                if (_callback_pvr_record_state_change) {
                    _callback_pvr_record_state_change(event, pData);
                }
                return;
                break;
            case this.ePvrEvent.PLAYBACK_POSITION:
                _currentTimeshiftTime = pData;
                _callback_play_record_state_change(event, pData);
                break;
            case this.ePvrEvent.PLAYBACK_SPEED:
                _currentSpeed = pData / 100;
                if (_currentSpeed == 1) {
                    _PVRState = this.ePvrState.PLAYBACK;
                }
                else if (_currentSpeed == 0) {
                    _PVRState = this.ePvrState.PAUSE;
                }
                break;
            case this.ePvrEvent.PLAYBACK_STOP:
            case this.ePvrEvent.END_OF_FILE:
                _callback_play_record_state_change(event);
                _currentSpeed = 1;
                return;
                break;
            case this.ePvrEvent.TIMESHIFT_START:
                _currentSpeed = 1;
                _PVRState = this.ePvrState.PLAYBACK;
                break;
            case this.ePvrEvent.TIMESHIFT_STOP:
                _currentSpeed = 99;
                _PVRState = this.ePvrState.LIVE;
                break;
            case this.ePvrEvent.TIMESHIFT_POSITION:
                //pData - recordTime
                //pData2 - playbackTime
                //pData3 - (bool) transition to normal playback requested
                break;
            case this.ePvrEvent.TIMESHIFT_SPEED:
                _currentSpeed = pData / 100;
                if (_currentSpeed == 1) {
                    _PVRState = this.ePvrState.PLAYBACK;
                }
                else if (_currentSpeed == 0) {
                    _PVRState = this.ePvrState.PAUSE;
                }
                break;
            case this.ePvrEvent.RECORD_NOT_ALLOWED:
                if (_callback_pvr_record_state_change) {
                    _callback_pvr_record_state_change(event, pData);
                }
                return;
            case this.ePvrEvent.PLAYBACK_MEDIA_EXPIRED:
                if (_callback_play_record_state_change) {
                    _callback_play_record_state_change(event);
                }
                return;
            case this.ePvrEvent.PLAYBACK_SPEED_NOT_ALLOWED:
                //message is sent for PVR playback and for timeshift
                _PVRState = _PVRstate_prev;
                if (_callback_play_record_state_change) {
                    _callback_play_record_state_change(event);
                }
                if (_callback_pvr_function) {
                    _callback_pvr_function(0, event, 0, 0);
                }
                return;
            case this.ePvrEvent.PLAYBACK_SPEED_CHANGED:
                //TODO: remove this case and event from dbus
                return;
            case this.ePvrEvent.PREPARE_SCHEDULED_RECORDING:
                //pData - recordIndex
                //pData2 - progHandle
                //pData3 - isStarting (used to make a difference between recording start and 1min before notification)
                //TODO: Ask user if he wants to zap to serviceIndex in order to record the event that was scheduled
                //      If user selects YES for answer Zap to serviceIndex from listIndex and start recording with progHandle by calling fPVRRecordByProgHandle
                //		If user selects NO as answer delete the recording with progHandle
                _callback_pvr_scheduled_recording(pData, pData2, pData3);
                return;
            case this.ePvrEvent.PLAYBACK_EXPIRATION_DATE:
                if (_callback_play_record_state_change) {
                    _callback_play_record_state_change(event, expirationDate);
                }
                return;
            case this.ePvrEvent.LIMITED_TIMESHIFT:
                if (_callback_pvr_function) {
                    _callback_pvr_function(0, event, 0, 0);
                }
                return;
            case this.ePvrEvent.TIMESHIFT_NOT_ALLOWED:
                if (_callback_pvr_function) {
                    _callback_pvr_function(0, event, 0, 0);
                }
                return;
            case this.ePvrEvent.INCREASE_TIMESHIFT_NOT_ALLOWED:
                if (_callback_pvr_function) {
                    _callback_pvr_function(0, event, 0, 0);
                }
                return;
            case this.ePvrEvent.PENDING_SCHEDULED_RECORDING:
                //pData - recordIndex
                //pData2 - progHandle
                //pData3 - isStarting (used to make a difference between recording start and 1min before notification)
                // Notify user that recording will start in 1 min time.
                // Ask if he wants to zapp to service that will be recorded.
                _callback_pvr_scheduled_recording(pData, pData2, pData3);
                return;
            default:
                return; //not handling other events
        }
        _PVREvent = event;
        if (_callback_pvr_function) {
            //this is timeshift callback in UI
            _callback_pvr_function(_currentSpeed, event, pData, pData2, pData3);
        }
    };
    /**
     * Fills pvr items into array
     * @private
     * @method _fPvrFileCallback
     * @param index {Number} Index of PVR item
     */
    var _fPvrFileCallback = function (index) {
            try {
                var tmp;
                if (PluginInterface.Mappings.Globals.gPCTest) {
                    tmp = PluginInterface.Mappings.Dummies.Pvr.getPvrInfo(index);
                }
                else {
                    tmp = PluginInterface.Setup.stb.pvrInfo("" + index);
                }
                _pvrArray[index] = _getPVR(tmp, index);
                if (index < _pvrCount - 1) {
                    index++;
                    clearTimeout(_tPvrCallback);
                    _tPvrCallback = setTimeout(_fPvrFileCallback, 1, index);
                }
                else if (_callback_pvr_browsing_function) {
                    _callback_pvr_browsing_function(1);
                }
            }
            catch (e) {
                clearTimeout(_tPvrCallback);
            }
        },
        /**
         * Returns pvr event object
         * @private
         * @method _pvrEventObject
         * @param desc {String} Description
         * @returns {Object}
         */
        _pvrEventObject = function (desc) {
            var temp = desc.split("|");
            return {
                name: (temp[0] ? temp[0] : ""),
                description: (temp[1] ? temp[1] : ""),
                startt: (temp[2] ? parseInt(temp[2], 10) : 0),
                startdt: (temp[3] ? temp[3] : "0000:00:00:00:00:00"),
                serviceIndex: (temp[4] ? temp[4] : "0"),
                repeatType: (temp[5] ? temp[5] : "0"),
                lcnServiceIndex: (temp[6] ? temp[6] : "0"),
                lcnListIndex: (temp[7] ? temp[7] : "0"),
                endt: (temp[8] ? parseInt(temp[8], 10) : 0),
                enddt: (temp[9] ? temp[9] : "0000:00:00:00:00:00"),
                eventID: (temp[10] ? temp[10] : "0"),
                genre: (temp[11] ? temp[11] : ""),
                parental: (temp[12] ? temp[12] : "0"),
                extendedDescription: (temp[13] ? temp[13] : '')
            }
        };
    /*
     ***********************************************************************************************************
     PRIVILEGED PROPERTIES AND METHODS
     ***********************************************************************************************************
     */

    /**
     * Enumerated PVR states
     * @property ePvrState
     * @type {{LIVE: number, PAUSE: number, PLAYBACK: number, FAST_REWIND: number, FAST_FORWARD: number}}
     * @final
     */
    this.ePvrState = {
        LIVE: 0,
        PLAYBACK: 1,
        PAUSE: 2,
        FAST_REWIND: 3,
        FAST_FORWARD: 4
    };

    /**
     * Enumerated PVR events
     * @property ePvrEvent
     * @type {{RECORD_START: number, RECORD_STOP: number, RECORD_POSITION: number, RECORD_ADD: number, RECORD_RESOURCE_ISSUE: number, PLAYBACK_STOP: number, PLAYBACK_POSITION: number, PLAYBACK_SPEED: number, TIMESHIFT_START: number, TIMESHIFT_STOP: number, TIMESHIFT_POSITION: number, TIMESHIFT_SPEED: number, END_OF_FILE: number, RECORD_NOT_ALLOWED: number, PLAYBACK_MEDIA_EXPIRED: number, PLAYBACK_SPEED_NOT_ALLOWED: number, PLAYBACK_SPEED_CHANGED: number, PREPARE_SCHEDULED_RECORDING: number, PLAYBACK_EXPIRATION_DATE: number, LIMITED_TIMESHIFT: number, TIMESHIFT_NOT_ALLOWED: number, INCREASE_TIMESHIFT_NOT_ALLOWED: number, PENDING_SCHEDULED_RECORDING: number}}
     * @final
     */
    this.ePvrEvent = {
        RECORD_START: 1,
        RECORD_STOP: 2,
        RECORD_POSITION: 3,
        RECORD_ADD: 4,
        RECORD_RESOURCE_ISSUE: 7,
        PLAYBACK_STOP: 11,
        PLAYBACK_POSITION: 12,
        PLAYBACK_SPEED: 13,
        TIMESHIFT_START: 15,
        TIMESHIFT_STOP: 16,
        TIMESHIFT_POSITION: 17,
        TIMESHIFT_SPEED: 18,
        END_OF_FILE: 30,
        RECORD_NOT_ALLOWED: 31,
        PLAYBACK_MEDIA_EXPIRED: 32,
        PLAYBACK_SPEED_NOT_ALLOWED: 33,
        PLAYBACK_SPEED_CHANGED: 34,
        PREPARE_SCHEDULED_RECORDING: 35,
        PLAYBACK_EXPIRATION_DATE: 36,
        LIMITED_TIMESHIFT: 37,
        TIMESHIFT_NOT_ALLOWED: 38,
        INCREASE_TIMESHIFT_NOT_ALLOWED: 39,
        PENDING_SCHEDULED_RECORDING: 40
    };

    /**
     * Enumerated PVR padding values
     * @property ePvrPadding
     * @type {{PADDING_NONE: number, PADDING_2: number, PADDING_5: number, PADDING_10: number}}
     * @final
     */
    this.ePvrPadding = {
        PADDING_NONE: 0,
        PADDING_2: 2,
        PADDING_5: 5,
        PADDING_10: 10
    };

    /**
     * Enumerated delayed on touch stop recording values
     * @property eDelayedOnetouchStop
     * @type {{NOW: number, STOP_15: number, STOP_30: number}}
     * @final
     */
    this.eDelayedOnetouchStop = {
        NOW: 0,
        STOP_15: 15,
        STOP_30: 30
    };

    /**
     * Enumerated schedule recording statuses
     * @property eScheduleRecordingStatus
     * @type {{SUCCESS: number, ERROR: number, REMOVED: number}}
     * @final
     */
    this.eScheduleRecordingStatus = {
        SUCCESS: 0,
        ERROR: 1,
        REMOVED: 2
    };

    /**
     * Enumerated PVR source service lists
     * @property pvrSourceList
     * @type {{ALL: number, TV: number, RADIO: number}}
     * @final
     */
    this.pvrSourceList = {
        ALL: 0,
        TV: 1,
        RADIO: 2
    };

    /**
     * Enumerated timeshift resource issue state
     * @property eTimeshiftResourceStatus
     * @type {{STORAGE_REMOVED: number}}
     * @final
     */
    this.eTimeshiftResourceStatus = {
        STORAGE_REMOVED: 1
    };

    /**
     * Initializes PVR module
     * @method fInitPvr
     * @static
     */
    this.fInitPvr = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.pvrNotification = _fCallbackPvr.bind(this);
        PluginInterface.Setup.stb.registerPvrCallback("");
    };

    /**
     * Set Callbacks
     * @method fSetCallbacks
     * @param callback {Function} PVR callback function
     * @param pvr_browsing_clbck {Function} PVR browsing callback
     * @param pvr_rec_state_change_clbck {Function} PVR record state change callback
     * @param pvr_play_state_change_clbck {Function} PVR record state change callback
     * @static
     */
    this.fSetCallbacks = function (callback, pvr_browsing_clbck, pvr_rec_state_change_clbck, pvr_play_state_change_clbck, pvr_scheduled_recording_callback) {
        _callback_pvr_function = callback;
        _callback_pvr_browsing_function = pvr_browsing_clbck;
        _callback_pvr_record_state_change = pvr_rec_state_change_clbck;
        _callback_play_record_state_change = pvr_play_state_change_clbck;
        _callback_pvr_scheduled_recording = pvr_scheduled_recording_callback;
    };

    /**
     * Returns current PVR state
     * 0 = Live stream playing, 1 = Paused, 2 = Timeshifted (recorded) content playing.
     * @method fGetPVRState
     * @returns {Number} PVR state
     * @static
     */
    this.fGetPVRState = function () {
        return _PVRState;
    };

    /**
     * Reset current PVR speed
     * @method fResetPVRCurrentSpeed
     * @final
     */
    this.fResetPVRCurrentSpeed = function () {
        _currentSpeed = 1;
    };

    /**
     * Plays timeshifted (recorded) content
     * @method fPVRPlay
     * @returns {Boolean} Play pvr recording status
     * @static
     */
    this.fPVRPlay = function (index) {
        var result = true;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _PVRState = this.ePvrState.PLAYBACK;
            _PVRstate_prev = this.ePvrState.PLAYBACK;
            _fCallbackPvr.call(this, 0, 30, 0);
            _currentTimeshiftTime = 0;
            return result;
        }
        else {
            if (typeof index == 'undefined') {
                this.fPVRResetSpeed();
                result = PluginInterface.Setup.stb.pvrPlay("");
                if (result) {
                    _PVRState = this.ePvrState.PLAYBACK;
                    _PVRstate_prev = this.ePvrState.PLAYBACK;
                }
                else {
                    PluginInterface.Epg.fRegisterEpgEventCallback();
                    _PVRState = this.ePvrState.LIVE;
                    _PVRstate_prev = this.ePvrState.LIVE;
                }
                _currentTimeshiftTime = 0;
                return result;
            }
            else {
                this.fRealPvrPlay(index);
            }
        }
    };
    /**
     * Stops displaying of timeshifted content and starts displaying of live stream
     * @method fPVRStop
     * @returns {Boolean} Stop pvr recording status
     * @static
     */
    this.fPVRStop = function () {
        _PVRState = this.ePvrState.LIVE;
        _PVRstate_prev = this.ePvrState.LIVE;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _fCallbackPvr.call(this, 0, 33, 0);
            return false;
        }
        return PluginInterface.Setup.stb.pvrStop("");
    };

    /**
     * Start timeshift recording
     * @method fPVRRecord
     * @returns {Boolean} Start timeshift recording status
     * @static
     */
    this.fPVRRecord = function () {
        var result = false;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _PVRState = this.ePvrState.LIVE;
            _PVRstate_prev = this.ePvrState.LIVE;
            result = true;
        }
        else {
            if (_transitionToRecord == 1) {
                return true;
            }
            result = PluginInterface.Setup.stb.pvrRecord("");
            if (result) {
                _transitionToRecord = 1;
            }
            _PVRState = this.ePvrState.LIVE;
            _PVRstate_prev = this.ePvrState.LIVE;
        }
        return result;
    };

    /**
     * Start pvr recording by prog handle
     * @method fPVRRecordByProgHandle
     * @returns {Boolean}
     * @static
     */
    this.fPVRRecordByProgHandle = function (progHandle, listIndex, serviceIndex) {
        var result = false;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _PVRState = this.ePvrState.LIVE;
            _PVRstate_prev = this.ePvrState.LIVE;
            result = true;
        }
        else {
            if (_transitionToRecord == 1) {
                return true;
            }
            result = PluginInterface.Setup.stb.pvrRecordByProgHandle("" + progHandle, "" + listIndex, "" + serviceIndex);
            if (result) {
                _transitionToRecord = 1;
            }
            _PVRState = this.ePvrState.LIVE;
            _PVRstate_prev = this.ePvrState.LIVE;
        }
        return result;
    };

    /**
     * Pauses live stream video
     * Freezed picture is displayed and recording of live stream is started
     * @method fPVRPause
     * @returns {Boolean}
     * @static
     */
    this.fPVRPause = function () {
        var result = false;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _PVRState = this.ePvrState.PAUSE;
            _PVRstate_prev = this.ePvrState.PAUSE;
            result = true;
            _fCallbackPvr.call(this, 0, 30, 0);
        }
        else {
            // if we are in live or timeshift playback call pause in plugin
            if ((_PVRState == this.ePvrState.LIVE) || (_PVRState >= this.ePvrState.PLAYBACK)) {
                PluginInterface.Hbbtv.fSetHbbtvOff();
                result = PluginInterface.Setup.stb.pvrPause("");
                if (result == true) {
                    _PVRState = this.ePvrState.PAUSE;
                    _PVRstate_prev = this.ePvrState.PAUSE;
                }
                else {
                    PluginInterface.Epg.fRegisterEpgEventCallback();
                    PluginInterface.Hbbtv.fSetHbbtvOn();
                    _PVRstate_prev = this.ePvrState.LIVE;
                }
            }
            /* if we are already in paused state, just return true. */
            else {
                result = true;
            }
        }
        return result;
    };

    /**
     * Increases speed of timeshift playback
     * @method fPVRFastForward
     * @returns {Boolean}
     * @static
     */
    this.fPVRFastForward = function () {
        var ret = false;
        if (!PluginInterface.Mappings.Globals.gPCTest) {
            if (_currentSpeed <= 8) {
                ret = PluginInterface.Setup.stb.pvrFastForward("" + 2 * _currentSpeed);
                if (ret == true) {
                    _PVRstate_prev = _PVRState;
                    _PVRState = this.ePvrState.FAST_FORWARD;
                }
            }
            else {
                ret = PluginInterface.Setup.stb.pvrFastForward("" + 1);
                if (ret == true) {
                    _PVRstate_prev = _PVRState;
                    _PVRState = this.ePvrState.PLAYBACK;
                }
                _currentSpeed = 1;
            }
        }
        else {
            _PVRstate_prev = _PVRState;
            _PVRState = this.ePvrState.FAST_FORWARD;
            ret = true;
            _fCallbackPvr.call(this, 0, 30, 0);
        }
        return ret;
    };

    /**
     * Set Fast Forward/Rewind
     * @param speed {Number} Fast Forward/Rewind speed
     * @returns {Boolean}
     * @static
     */
    this.fSetForwardRewind = function (speed) {
        _currentSpeed = speed;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            if (speed === 1) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.PLAYBACK;
            }
            else if (speed > 1) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.FAST_FORWARD;
            }
            else {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.FAST_REWIND;
            }
            return true;
        }

        if (speed === 1) {
            if (PluginInterface.Setup.stb.pvrFastForward("" + speed)) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.PLAYBACK;
                return true;
            }
        }
        else if (speed > 1) {
            if (PluginInterface.Setup.stb.pvrFastForward("" + speed)) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.FAST_FORWARD;
                return true;
            }
        }
        else {
            if (PluginInterface.Setup.stb.pvrRewind("" + Math.abs(speed))) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.FAST_REWIND;
                return true;
            }
        }
        return false;
    };

    /**
     * Reset PVR speed
     * @method fPVRResetSpeed
     * static
     */
    this.fPVRResetSpeed = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _PVRstate_prev = _PVRState;
            _PVRState = this.ePvrState.PLAYBACK;
            _currentSpeed = 1;
            return;
        }
        if (_currentSpeed != 1) {
            var ret = PluginInterface.Setup.stb.pvrFastForward("" + 1);
            if (ret == true) {
                _PVRstate_prev = _PVRState;
                _PVRState = this.ePvrState.PLAYBACK;
            }
            _currentSpeed = 1;
        }
    };

    /**
     * Get current speed of PVR playback
     * @method fPVRGetCurrentSpeed
     * @returns {number}
     * static
     */
    this.fPVRGetCurrentSpeed = function () {
        return _currentSpeed;
    };

    /**
     * Decreases speed of timeshift playback
     * @method fPVRRewind
     * @returns {boolean}
     * @static
     */
    this.fPVRRewind = function () {
        var ret = false;
        if (!PluginInterface.Mappings.Globals.gPCTest) {
            if (_currentSpeed >= -8) {
                if (_currentSpeed > 0) {
                    ret = PluginInterface.Setup.stb.pvrRewind("" + 1);
                }
                else {
                    ret = PluginInterface.Setup.stb.pvrRewind("" + (-2) * _currentSpeed);
                }
                if (ret) {
                    _PVRstate_prev = _PVRState;
                    _PVRState = this.ePvrState.FAST_REWIND;
                }
            }
            else {
                ret = PluginInterface.Setup.stb.pvrFastForward("" + 1);
                if (ret == true) {
                    _PVRstate_prev = _PVRState;
                    _PVRState = this.ePvrState.PLAYBACK;
                }
                _currentSpeed = 1;
            }
        }
        else {
            _PVRstate_prev = _PVRState;
            _PVRState = this.ePvrState.FAST_REWIND;
            ret = true;
            _fCallbackPvr.call(this, 0, 30, 0);
        }
        return ret;
    };

    /**
     * Gets duration of recorded stream
     * @method fPVRGetDuration
     * @returns {Number} Duration of recorded stream
     * @static
     */
    this.fPVRGetDuration = function () {
        return _endTimeshiftTime;
    };

    /**
     * Gets current time of recorded stream
     * @method fPVRGetCurrentTime
     * @returns {Number} Current time of recorded stream
     * @static
     */
    this.fPVRGetCurrentTime = function () {
        return _currentTimeshiftTime;
    };

    /**
     * Returns number of pvr items
     * @method fGetCurrPvrNumber
     * @returns {Number} Number of PVR items
     * @static
     */
    this.fGetCurrPvrNumber = function () {
        return _pvrCount;
    };

    /**
     * Returns previous index of pvr items
     * @method fGetPreviousPvrIndex
     * @param index {Number} Index of PVR item
     * @returns {Number} Previous index of pvr items
     * @static
     */
    this.fGetPreviousPvrIndex = function (index) {
        if (index > _pvrCount - 1) {
            return -1;
        }
        if (index == 0) {
            return _pvrCount - 1;
        }
        return index - 1;
    };

    /**
     * Returns next index of pvr items
     * @method fGetNextPvrIndex
     * @param index {Number} Index of PVR item
     * @returns {Number} Next index of pvr items
     * @static
     */
    this.fGetNextPvrIndex = function (index) {
        if (index > _pvrCount - 1) {
            return -1;
        }
        if (index == _pvrCount - 1) {
            return 0;
        }
        return index + 1;
    };

    /**
     * Returns pvr item object
     * @method fGetPvrInfo
     * @param index {Number} Index of PVR item
     * @returns {Object} PVR item object
     * @static
     */
    this.fGetPvrInfo = function (index) {
        return _pvrArray[index];
    };

    /**
     * Start loading PVR items into array
     * @method fPvrBrowse
     * @returns {Boolean} PVR brovse status
     * @static
     */
    this.fPvrBrowse = function () {
        _pvrArray = [];
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _pvrCount = 25;
        }
        else {
            _pvrCount = PluginInterface.Setup.stb.pvrRecNum("");
        }

        if (_pvrCount > 0) {
            _fPvrFileCallback(0);
        }
        else if (_callback_pvr_browsing_function) {
            _callback_pvr_browsing_function(1);
        }
        return (_pvrCount > 0);
    };

    /**
     * Plays pvr content
     * @method fRealPvrPlay
     * @param index {Number} Index of PVR content
     * @returns {Number} 1 = success, 0 = error
     * @static
     */
    this.fRealPvrPlay = function (index) {
        _currentPvrPlayingIndex = index;
        if (!PluginInterface.Mappings.Globals.gPCTest) {
            PluginInterface.Setup.stb.pvrStartPlay("" + index);
        }
        _currentTimeshiftTime = 0;
        _PVRstate_prev = this.ePvrState.PLAYBACK;
        _PVRState = this.ePvrState.PLAYBACK;
    };

    /**
     * Stop pvr content playback
     * @method fRealPvrStop
     * @static
     */
    this.fRealPvrStop = function () {
        _currentPvrPlayingIndex = -1;
        _currentSpeed = 1;
        _PVRState = this.ePvrState.LIVE;
        _PVRstate_prev = this.ePvrState.LIVE;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _callback_play_record_state_change(this.ePvrEvent.PLAYBACK_STOP);
            _currentSpeed = 1;
            return;
        }
        PluginInterface.Setup.stb.pvrStopPlay("");
    };

    /**
     * Returns current playing index
     * @method fGetPlayingPvrIndex
     * @returns {Number} Current playing index
     * @static
     */
    this.fGetPlayingPvrIndex = function () {
        return _currentPvrPlayingIndex;
    };

    /**
     * Returns number of pvr files
     * @method fGetPvrCountIndex
     * @returns {Number} Count of pvr files
     * @static
     */
    this.fGetPvrCount = function () {
        return _pvrArray.length;
    };

    /**
     * Set PVR path
     * @method fSetPVRPath
     * @param path {String} PVR path
     * @static
     */
    this.fSetPVRPath = function (path) {
        if (!PluginInterface.Mappings.Globals.gPCTest) {
            PluginInterface.Setup.stb.pvrSetDevicePath(path);
        }
    };

    /**
     * Returns if TS is active or not
     * @method fGetPvrIsTimeshiftActive
     * @returns {Number} 1 == True, 0 == False
     * @static
     */
    this.fPvrIsTimeshiftActive = function () {
        if (!PluginInterface.Mappings.Globals.gPCTest) {
            return PluginInterface.Setup.stb.pvrIsTimeshiftActive();
        }
    };

    /**
     * Create timer record
     * @method fCreateTimerRecord
     * @param recordData {String} record data
     * @static
     */
    this.fCreateTimerRecord = function (recordData, index, list) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return 0;
        }
        return PluginInterface.Setup.stb.createTimerRecord("" + recordData, "" + index, "" + list);
    };

    /**
     * Delete pvr media
     * @method fPvrDeleteMedia
     * @param index {Number} index of pvr file
     * @static
     */
    this.fPvrDeleteMedia = function (index) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return 0;
        }
        return PluginInterface.Setup.stb.pvrDeleteMedia("" + index);
    };

    /**
     * Get scheduled record index
     * @method fGetScheduledRecordIndex
     * @returns {Number} Scheduled record index
     * @static
     */
    this.fGetScheduledRecordIndex = function () {
        return _scheduledRecordIndex;
    };

    /**
     * Get number of scheduled records
     * @method fGetNumberOfScheduledRecords
     * @returns {Number} Number of scheduled records
     * @static
     */
    this.fGetNumberOfScheduledRecords = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _schedulesCount;
        }
        return PluginInterface.Setup.stb.pvrUpdateRecordList("");
    };

    /**
     * Get scheduled record
     * @method fGetScheduledRecord
     * @param index {Number}
     * @returns {Object} Scheduled record
     * @static
     */
    this.fGetScheduledRecord = function (index) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            var eventDuration = 120; // sec
            var dateTime = new Date().getTime();
            var startt = dateTime / 1000;
            var startDateTime = new Date(startt * 1000);
            var startdt = '2012:09:25:' + ((startDateTime.getHours() < 10) ? '0' + startDateTime.getHours() : startDateTime.getHours()) + ':' + ((startDateTime.getMinutes() < 10) ? '0' + startDateTime.getMinutes() : startDateTime.getMinutes()) + ':' + startDateTime.getSeconds();
            var endt = (dateTime / 1000) + eventDuration;
            var endDateTime = new Date(endt * 1000);
            var enddt = '2012:09:25:' + endDateTime.getHours() + ':' + endDateTime.getMinutes() + ':' + endDateTime.getSeconds();
            return _pvrEventObject("Event - " + index + "|Event description|" + startt + "|" + startdt + "|0|0|1|6|" + endt + "|" + enddt);
        }
        return _pvrEventObject(PluginInterface.Setup.stb.pvrGetRecordInfo("" + index));
    };

    /**
     * Get record type
     * @method fGetRecordType
     * @param index
     * @returns {Number} Record type
     * @static
     */
    this.fGetRecordType = function (index) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return 0;
        }
        return PluginInterface.Setup.stb.pvrGetRecordType("" + index);
    };

    /**
     * Destroy scheduled record
     * @method fDestroyScheduledRecord
     * @param index {Number} Scheduled record index
     * @static
     */
    this.fDestroyScheduledRecord = function (index) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            PluginInterface.Tools.debug("Destroy scheduled record with index: " + index);
            _schedulesCount--;
            return;
        }
        PluginInterface.Setup.stb.pvrDestroyRecord("" + index);
    };

    /**
     * Get recording channel index
     * @method fGetRecordingChannelIndex
     * @returns {Number} Recording channel index
     * @static
     */
    this.fGetRecordingChannelIndex = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return -1;
        }
        return PluginInterface.Setup.stb.getRecordingChannelIndex("");
    };

    /**
     * Set PVR padding before
     * @method fSetPvrPaddingBefore
     * @param value {Number} PVR padding before value
     * @static
     */
    this.fSetPvrPaddingBefore = function (value) {
        _pvrPaddingBefore = value;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.saveData(PluginInterface.Setup.DATA_PVR_PADDING_BEFORE, "" + value);
    };

    /**
     * Get PVR padding before
     * @method fGetPvrPaddingBefore
     * @returns {Number} PVR padding before value
     * @static
     */
    this.fGetPvrPaddingBefore = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _pvrPaddingBefore;
        }
        var value = parseInt(PluginInterface.Setup.stb.loadData(PluginInterface.Setup.DATA_PVR_PADDING_BEFORE), 10);
        _pvrPaddingBefore = (typeof value == 'undefined' || isNaN(value)) ? this.ePvrPadding.PADDING_NONE : value;
        return _pvrPaddingBefore;
    };

    /**
     * Set PVR padding after
     * @method fSetPvrPaddingAfter
     * @param value {Number} PVR padding after value
     * @static
     */
    this.fSetPvrPaddingAfter = function (value) {
        _pvrPaddingAfter = value;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.saveData(PluginInterface.Setup.DATA_PVR_PADDING_AFTER, "" + value);
    };

    /**
     * Get PVR padding after
     * @method fGetPvrPaddingAfter
     * @returns {Number} PVR padding after value
     * @static
     */
    this.fGetPvrPaddingAfter = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _pvrPaddingAfter;
        }
        var value = parseInt(PluginInterface.Setup.stb.loadData(PluginInterface.Setup.DATA_PVR_PADDING_AFTER), 10);
        _pvrPaddingAfter = (typeof value == 'undefined' || isNaN(value)) ? this.ePvrPadding.PADDING_NONE : value;
        return _pvrPaddingAfter;
    };

    /**
     * Set title of the recorded item
     * @method fSetRecordedTitle
     * @param index {Number} Index od recording item
     * @param title {String} Title of recording item
     * @static
     */
    this.fSetRecordedTitle = function (index, title) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.setRecordedTitle("" + index, "" + title);
    };

    /**
     * Set pvr buffer size in seconds
     * @method PvrSetBufferIntervalInSec
     * @param bufferSize {Number} Buffer size in seconds
     * @static
     */
    this.PvrSetBufferIntervalInSec = function (bufferSize) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.pvrSetBufferIntervalInSec("" + bufferSize);
    }
};